@@ -7,7 +7,7 @@ RDF::Trine - An RDF Framework for Perl
 
 =head1 VERSION
 
-This document describes RDF::Trine version 0.128
+This document describes RDF::Trine version 0.129
 
 =head1 SYNOPSIS
 
@@ -39,6 +39,7 @@ components:
 
 package RDF::Trine;
 
+use 5.008003;
 use strict;
 use warnings;
 no warnings 'redefine';
@@ -46,13 +47,16 @@ no warnings 'redefine';
 our ($debug, @ISA, $VERSION, @EXPORT_OK);
 BEGIN {
 	$debug		= 0;
-	$VERSION	= '0.128';
+	$VERSION	= '0.129';
 	
 	require Exporter;
 	@ISA		= qw(Exporter);
-	@EXPORT_OK	= qw(iri blank literal variable statement store);
+	@EXPORT_OK	= qw(iri blank literal variable statement store UNION_GRAPH NIL_GRAPH);
 }
 
+use constant UNION_GRAPH	=> 'tag:gwilliams@cpan.org,2010-01-01:RT:ALL';
+use constant NIL_GRAPH		=> 'tag:gwilliams@cpan.org,2010-01-01:RT:NIL';
+
 use Log::Log4perl qw(:easy);
 Log::Log4perl->easy_init($ERROR);
 
