@@ -7,7 +7,7 @@ RDF::Trine::Model - Model class
 
 =head1 VERSION
 
-This document describes RDF::Trine::Model version 0.128
+This document describes RDF::Trine::Model version 0.129
 
 =head1 METHODS
 
@@ -23,7 +23,7 @@ no warnings 'redefine';
 
 our ($VERSION);
 BEGIN {
-	$VERSION	= '0.128';
+	$VERSION	= '0.129';
 }
 
 use Scalar::Util qw(blessed);
@@ -72,7 +72,7 @@ sub temporary_model {
 # 	my $store	= RDF::Trine::Store::DBI->temporary_store();
 	my $self	= $class->new( $store );
 	$self->{temporary}	= 1;
-	$self->{threshold}	= 2000;
+	$self->{threshold}	= 25_000;
 	return $self;
 }
 
@@ -136,11 +136,19 @@ sub add_statement {
 		if ($self->{added}++ >= $self->{threshold}) {
 # 			warn "*** should upgrade to a DBI store here";
 			my $store	= RDF::Trine::Store::DBI->temporary_store;
-			my $iter	= $self->get_statements();
+			my $iter	= $self->get_statements(undef, undef, undef, undef);
+			if ($store->can('_begin_bulk_ops')) {
+				$store->_begin_bulk_ops();
+			}
 			while (my $st = $iter->next) {
 				$store->add_statement( $st );
 			}
+			if ($store->can('_end_bulk_ops')) {
+				$store->_end_bulk_ops();
+			}
 			$self->{store}	= $store;
+			$self->{temporary}	= 0;
+# 			warn "*** upgraded to a DBI store";
 		}
 	}
 	return $self->_store->add_statement( @_ );
@@ -263,6 +271,13 @@ predicate and objects. Any of the arguments may be undef to match any value.
 sub count_statements {
 	my $self	= shift;
 	$self->end_bulk_ops();
+
+	if (scalar(@_) >= 4) {
+		my $graph	= $_[3];
+		if (blessed($graph) and $graph->isa('RDF::Trine::Node::Resource') and $graph->uri_value eq 'tag:gwilliams@cpan.org,2010-01-01:RT:ALL') {
+			$_[3]	= undef;
+		}
+	}
 	return $self->_store->count_statements( @_ );
 }
 
@@ -283,6 +298,13 @@ the underlying store).
 sub get_statements {
 	my $self	= shift;
 	$self->end_bulk_ops();
+	
+	if (scalar(@_) >= 4) {
+		my $graph	= $_[3];
+		if (blessed($graph) and $graph->isa('RDF::Trine::Node::Resource') and $graph->uri_value eq 'tag:gwilliams@cpan.org,2010-01-01:RT:ALL') {
+			$_[3]	= undef;
+		}
+	}
 	return $self->_store->get_statements( @_ );
 }
 
@@ -715,12 +737,24 @@ sub _store {
 
 sub _debug {
 	my $self	= shift;
+	my $warn	= shift;
 	my $stream	= $self->get_statements( undef, undef, undef, undef );
 	my $l		= Log::Log4perl->get_logger("rdf.trine.model");
 	$l->debug( 'model statements:' );
+	if ($warn) {
+		warn "Model $self:\n";
+	}
+	my $count	= 0;
 	while (my $s = $stream->next) {
+		$count++;
+		if ($warn) {
+			warn $s->as_string . "\n";
+		}
 		$l->debug('[model]' . $s->as_string);
 	}
+	if ($warn) {
+		warn "$count statements\n";
+	}
 }
 
 1;
